; LAML, LENO, and ELUCIDATOR template support and other useful editing commands on LAML files.

; The LAML library and programs written by Kurt Normark, Aalborg University, Denmark.
; Copyright (C) 1999  Kurt Normark, normark@cs.auc.dk.
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

; Top level functions in this package
; insert-f for each LENO function f. 
; Also insert manual page/section, and elucidator templates.
; The functions embed, nest and unnest are also very useful (in general).

; Notice: note-page templates are supported via the general template facily,
; which is now part of this file.


; This variable depends on the variable laml-scheme-dir, which is supposed to be defined
; in the emacs init file (emacs.init or .emacs)

(defvar laml-elisp-dir (concat laml-dir "emacs-support/") "The directory which contains this file. Ends in a slash")

(defvar laml-pp-helpful t "If t always pretty print role strings in templates. If nil, just print empty strings for some template elements")
(setq laml-pp-helpful t)

; ---------------------------------------------------------------------------------------------------
; General template support for emacs:


(defvar laml-template-dir (concat laml-elisp-dir "templates")
   "The directory path to the
directory, in which the templates (tpl as well as el) files are stored.")

(defun laml-insert-template (tmpl-name)
  "Inserts a template from the template collection in the directory
templates."
  (interactive 
    (list
      (completing-read
       "Template name: "
       (mapcar 'laml-tmpl-transform-name
               (filter 'laml-tmpl-tpl-filter
                       (file-name-all-completions "" laml-template-dir)))
             nil t nil)))
  (insert-file (concat laml-template-dir "/" tmpl-name ".tpl"))
  (let ((el-file (concat laml-template-dir "/" tmpl-name ".el")))
  (if (file-exists-p el-file)
      (load-file el-file))))

(defun insert-laml-template ()
    "Inserts a template from the template collection in the directory
templates. Identical to laml-insert-template, and just for convenience"
  (interactive)
  (call-interactively 'laml-insert-template))

;; Functions that help generate a completion list of template names:

(defun laml-tmpl-transform-name (n)
  "Function used for the production of the template name completion list"
  (cons (substring n 0 (- (length n) 4)) nil))

(defun laml-tmpl-tpl-filter (n)
  (equal (substring n (- (length n) 4)) ".tpl"))


; ---------------------------------------------------------------------------------------------------
; Template insertion
  
(defun insert-lisp-form (form indentation)
  (setq indent-tabs-mode nil)
  (ensure-indentation 0)
  (insert (concat (make-string indentation 32) form))
  (backward-sexp 1)
  (indent-sexp)
  (forward-sexp 1) (beginning-of-line)
  (ensure-indentation indentation)
  (end-of-line)
  (backward-sexp 1))

; old, but still used
(defun insert-lisp-form-0 (form indentation)
  (insert-lisp-form (laml-transliterate form) indentation))


; ---------------------------------------------------------------------------------------------------
; Transliteration stuff:

(defvar empty-string "\"\"")

(defun laml-transliterate (form)
  "Return a copy with various transliterations" ; not used
  (let* ((res1 (string-to-list form))
         (res2 (if (eq (car res1) 40)
                   (cons 40 (transliterate-list (cdr res1) 40 (concat CR "(")))  ; avoid CR before first start parenthesis
                   (transliterate-list res1 40 (concat CR "("))))
         (res3 (transliterate-list res2 41 (concat ")" CR)))
         (res4 (transliterate-list res3 ?E (concat empty-string CR)))
        )
    (list-to-string res4)))


(defun transliterate-list (list char x)
  "List is a list of characters (integers). Replaces occurences of char with x.
The parameter x can be a string or a character"
  (let ((list1 list)
        (res nil))
    (while list1 
      (setq res
        (cons 
          (if (eq (car list1) char)
              x
              (car list1))
          res))
      (setq list1 (cdr list1)))
    (reverse res)))


(defun charlist-to-string (lst)
  "The opposite of string-to-list"
  (let ((str (make-string (length lst) 0))
        (lst1 lst)
        (i 0))
    (while lst1 
       (aset str i (car lst1))
       (setq i (+ i 1))
       (setq lst1 (cdr lst1)))
    str))

(defun list-to-string (lst)
  "Convert a list of mixed chars and strings to a string"
  (let ((str (make-string (length-char-string-list lst) 0))
        (lst1 lst)
        (i 0))
    (while lst1 
       (aset-char-string str i (car lst1))
       (setq i (+ i (length-char-string (car lst1))))
       (setq lst1 (cdr lst1)))
    str))

(defun length-char-string-list (lst)
  "Calculate the length of lst. Chars count 1, strings count their lengths"
 (let ((res 0)
       (lst1 lst))
  (while lst1
    (setq res (+ res (length-char-string (car lst1))))
    (setq lst1 (cdr lst1)))
  res))

(defun length-char-string (x)
  "Return the length of x. Chars count 1, strings count their lengths"
  (cond ((integerp x) 1)
        ((stringp x) (length x))))

(defun aset-char-string (str i x)
  "like aset, but x may also be a string, in which case all chars in x are inserted"
  (cond ((integerp x) (aset str i x))
        ((stringp x) (aset-string str i x 0))))

(defun aset-string (str i str1 j)
  "Assign destructively str1 into position i in str. j is a helping, indexing parameter"
  (if (= j (length str1))
      nil  ; result in str
      (progn 
        (aset str i (aref str1 j))
        (aset-string str (+ i 1) str1 (+ j 1)))))
          


; ---------------------------------------------------------------------------------------------------
; Indentation stuff:

(defun determine-indentation ()
  (let ((ci (real-indentation-of-string (this-line))))
    (if (> ci 0)
        ci
        (indentation-of-previous-line))))

   
(defun indentation-of-previous-line ()
  "Return the indentation of the previous line. Here previous line means the previous
real line. Empty lines are not taken into consideration. In case no previous line exist, return 0"
  (save-excursion
    (beginning-of-line)
    (let ((prev (prev-line)))
      (if prev
          (cond ((space-stringp prev) (progn (previous-line 1) (indentation-of-previous-line)))
                (t (indentation-of-string prev)))
          0))))

(defun real-indentation-of-string (str)
  (cond ((equal str "") 0)
	((space-stringp str) (length str))
	(t (let ((i 0))
	     (while (eq (aref str i) 32)
	       (setq i (+ i 1)))
	     i))))

(defun indentation-of-string (str)
  "Return the number of leadning spaces of str. 
If string is empty, or if it consists soly of spaces, return 0"
 (if (or (equal str "") (space-stringp str))
     0
     (let ((i 0))
       (while (eq (aref str i) 32)
          (setq i (+ i 1)))
       i)))

(defun space-stringp (str)
  "Is str full of spaces"
  (or (= 0 (length str)) 
      (equal str (make-string (length str) 32))
  ))

(defun prev-line ()
 (save-excursion
  (let ((p1 (point)))
    (previous-line 1)
    (if (eq p1 (point))
        nil
        (this-line)))))

(defun this-line ()
  (save-excursion 
    (beginning-of-line)
    (let ((p1 (point)))
      (end-of-line)
      (buffer-substring p1 (point)))))    
     
(defun ensure-indentation (i)
  "Enforce that the current line is idented i positions."
  (save-excursion
   (let ((ci (real-indentation-of-string (this-line))))
      (if (not (= i ci))
          (progn
             (beginning-of-line)
             (delete-char ci)
             (insert-char 32 i))))))


; ---------------------------------------------------------------------------------------------------
; File making from template.
; (Originally located in elucidator.el)

(defun make-a-file-from-laml-template (file dir template-name emacs-mode replacements)
  "Make the FILE in DIR. Then insert the LAML template TEMPLATE-NAME. 
Finally carry out the list of REPLACEMENTS. Return the files buffer"
  (let ((new-buffer (create-file-buffer (concat dir file))))
    (set-buffer new-buffer)
    (eval (list 'quote emacs-mode)) ; set the mode
    (laml-insert-template template-name)
    (do-perform-replacements replacements)
    (write-file (concat dir file))
    new-buffer))

(defun do-perform-replacements (replacement-list)
  (mapcar
    (function do-perform-replacement)
    replacement-list))

(defun do-perform-replacement (repl)
  (let ((from-string (car repl))
        (to-string (cadr repl)))
    (goto-char (point-min))
    (while (search-forward from-string nil t)
      (replace-match to-string t t))))


(defun ensure-trailing-slash (dir-string)
  "Ensure that dir-string ends in a slash: /"
  (let* ((dir-lgt (length dir-string))
         (last-chr (aref dir-string (- dir-lgt 1))))
   (if (eq last-chr ?/)
       dir-string
       (concat dir-string "/"))))
  
  
; move to general.el (also in making-directories).
(defun determine-directory ()
  "What is the current directory"
  (cond ((buffer-file-name) (current-directory))
        ((eq major-mode 'dired-mode) (dired-current-directory))
        (t nil)))
             

; ---------------------------------------------------------------------------------------------------
; Templates
; LENO specific

(defvar leno-templates nil "LAML template list")

(setq leno-templates
  (list
   '(title (empty "title") (empty "comment"))

   '(text (empty "text") (empty "comment"))

   '(point (empty "the-point") (empty "comment"))

   ; items and item: special

   '(source-program
     "file-name" 
     (list "from-mark" "to-mark")
     (list
      (list "start" "end" color face)
      )
     (list (quote slide-inline) (quote book-external))
     "source file comment")

   '(image "file-name" "comment" (list (quote vertical) (quote picture)))

   '(example (empty "the-example") (empty "comment"))

   ; opposing: special

   '(syntax (empty "the-syntax") (empty "comment"))

   '(slide-space)

   '(tabular 1 width-list list-of-list "comment")

   '(note-text (empty "text"))     

   '(slide-text (empty "text"))     

   '(cross-references 
      (list reference-items)
    )

   '(internet-reference (empty "reference-text") "url")

   '(note-reference (empty "reference-text") lecture-id page-id)

   '(informal-reference "reference-text" "reference")

   '(exercise id "title" (formulation "formulation") (solution "solution"))

   '(long-slide)

   '(index-words (empty "word1") (empty "word2"))

   '(image-series "description" list-of-image-descriptions (list (quote slide-external) (quote book-inline)))

   '(section-title (empty "title"))

   '(slide-image "file-name")

   ; concept-list: special

   '(applet-program "class-file-name" "code-base" "explanation" width height)

   ; synopsis: special

   '(quotation "quotation-text" "commment")

   ))

; ---------------------------------------------------------------------------------------------------
; Template pretty printing:

(defun pretty-print-template (x cr-after-1 cr-after-2)
  "x is a list. Return a string which is the pretty printing of x. 
Cr-after-1 is the primary separator. Cr-after-2 is the last separator in a list. Both are symbols."
 (let ((x1 (butlast x))
       (x2 (last x)))
  (concat 
     "(" 
     (concat (apply (function concat)
                    (append
                      (mapcar (function (lambda(e) (pretty-print-element e cr-after-1))) x1)
                      (mapcar (function (lambda(e) (pretty-print-element e cr-after-2))) x2))
             ) 
             " ")
     ")")))

(defun pretty-print-element (y cr-after)
 "LAML pretty prints a single element"
 (concat
   (cond ((symbolp y) (symbol-to-string y))
         ((stringp y) (string-it y))
         ((integerp y) (int-to-string y))
         ((and (listp y) (> (length y) 0) (eq 'empty (car y))) (pretty-print-element (if laml-pp-helpful (cadr y) "") 'nothing))
         ((and (listp y) (> (length y) 0) (eq 'quote (car y))) (concat QUOTE (pretty-print-element (cadr y) cr-after)))
         ((and (listp y) (> (length y) 0) (eq 'list (car y))) (pretty-print-template y 'space 'nothing))
         ((and (listp y)) (pretty-print-template y cr-after cr-after))
         (t "??")) (space-element y cr-after)))

(defun space-element (y cr-after)
  "Returns the space element (a string) given an element and an separator symbol"
  (cond ((eq cr-after 'nothing) "")
        ((eq cr-after 'space) " ")
        ((eq cr-after 'cr) CR)
        (t " ")))

(defun laml-insert (kind)
 "The near top level laml template insertion form. Kind is a symbol, one of the cars of the template elements in templates"
 (let ((template (assq kind leno-templates)))
    (insert-lisp-form (pretty-print-template template 'cr 'cr) (determine-indentation))))
    

; -----------------------------------------------------------------------------
; Generation of template insertion functions

 (defun define-insertion-function (tag)
 "Defines a elist function (vial eval) for tag, which must be a symbol"
 (let ((string-tag (symbol-to-string tag)))
  (eval (list 'defun (string-to-symbol (concat "insert-" string-tag)) '() (list 'interactive)
          (list 'laml-insert (list 'quote tag))))))

; Programmatically defines insertion functions for all templates in the list leno-templates
(mapcar (function (lambda (template) (define-insertion-function (car template)))) leno-templates)
 

; ---------------------------------------------------------------------------------------------------
; Special template insertions functions (which cannot be made from the template list above):
; Defined the hard way, and using a special version of insert-lisp-form, which in turn
; uses the transliteration stuff in order to make line separation, and in order to provided
; for pretty printing via indent-sexp.
; LENO specific

(defun insert-items (n m)
  (interactive "nNumber of items at outer level: 
nNumber of items at the nested level: ")
  (insert-lisp-form-0 (make-items n m) (determine-indentation))
  (position-point-after "\"")
)


(defun insert-single-item (m)
 "Insert a single item with m subitems"
  (interactive "nNumber of subitems to the item: ")
  (insert-lisp-form-0 (make-item 1 m) (determine-indentation)))

(defun make-items (n m)
  (if (= n 0) 
      "" 
      (concat "(items " (make-item n m) ")"))
)

(defun make-item (n m)
  (if (= n 0) 
      ""
      (concat 
         (concat "(item E E" (make-items m 0) ")" (make-item (- n 1) m)))))



(defun insert-opposing (n)
  (interactive "nHow many opposing items: ")
  (insert-lisp-form-0
    (make-opposing n)
    (determine-indentation)))

(defun make-opposing (n)
  (concat
    "(opposing"
       (make-item n 0)
    ")" ))


(defun insert-concept-list (n)
  (interactive "nNumber of concepts: ")
  (insert-lisp-form-0 (make-concept-list n) (determine-indentation)))

(defun make-concept-list (n)
  (concat "(concept-list" (make-concept n)  ")"))

(defun make-concept (n)
  (if (= n 0) 
      ""
      (concat "(concept " "E E E)" (make-concept (- n 1)))))


(defun insert-synopsis (n)
  (interactive "nNumber of synopsis items: ")
  (insert-lisp-form-0
    (make-synopsis n)
    (determine-indentation)))


(defun make-synopsis (n)
  (if (= n 0) 
      "" 
      (concat "(synopsis " (make-synopsis-item n) ")"))
)

(defun make-synopsis-item (n)
  (if (= n 0) 
      ""
      (concat 
         (concat "(synopsis-item E E)" (make-synopsis-item (- n 1))))))

; ---------------------------------------------------------------------------------------------------
; LENO templates supported via text insertions from template directories

(defun position-point-after (char-str)
  (search-forward char-str nil t))

(defun goto-next-input-field ()
  (interactive)
  (position-point-after "\"\"")
  (backward-char 1))

(defun insert-notepage ()
  "Insert a LENO note page"
  (interactive)
  (laml-insert-template "notepage")
  (position-point-after "'")
)

(defun insert-notesection ()
  "Insert a LENO note section"
  (interactive)
  (laml-insert-template "notesection")
  (position-point-after "'")
)

(defun insert-leno-chapter ()
  (interactive)
  (insert-file (concat laml-dir "styles/lecture-notes/man/template-zip-dir/lecture-note-template.laml")))

(defun insert-leno-single-chapter ()
  (interactive)
  (insert-file (concat laml-dir "styles/lecture-notes/man/template-zip-dir/lecture-note-single-chapter-template.laml")))

; ---------------------------------------------------------------------------------------------------
; Creation of a LENO directory structure, and set up of initial LENO file.
; Modelled after the elucidator function make-eluciator

(defun make-lecture-notes (dir0 leno-file-name)
  "Make a single chapter LENO slide/lecture-note set in dir0."
  (interactive "DMake single chapter LENO slide set in which directory: 
sName of the lecture-note file (plain name without extension): 
")
  (let ((dir (ensure-trailing-slash (expand-file-name dir0)))
       )
   (if dir
       (if (or (file-exists-p (concat dir "html")) (file-exists-p (concat dir "internal")))
          (progn 
            (beep) 
            (message "A lecture note set already seems to be set up in this directory. Nothing done!"))
          (progn
            (make-directory (concat dir "html/"))
            (make-directory (concat dir "html/applets/"))
            (make-directory (concat dir "html/external-html/"))
            (make-directory (concat dir "graphics/"))
            (make-directory (concat dir "graphics/small"))
            (make-directory (concat dir "html/graphics/"))
            (make-directory (concat dir "html/graphics/small"))
            (make-directory (concat dir "html/images/"))
            (make-directory (concat dir "html/javascript/"))
            (make-directory (concat dir "internal/"))
            (make-directory (concat dir "includes/"))
            (make-directory (concat dir "html/lecturer-photos/"))
            (let* ((leno-buf nil))

              ; Make the setup file
              (setq leno-buf
                    (make-a-file-from-laml-template 
                     (concat leno-file-name ".laml")
                     dir
                     "leno-single-chapter"
                     'laml-mode
                       (list
                         (list "CHAPTER-ID" leno-file-name)
                       )
                     ))

              (show-buffer (selected-window) leno-buf)

              (message "Done. Now fill in the CAPITALIZED ENTITIES and start writing your material.")

              )))

    (progn
      (beep)
      (message "Cannot determine the current directory. Sorry...")))))


; ---------------------------------------------------------------------------------------------------

; String splitting stuff:

(defvar QUOTE-CHAR 34)
(defvar QUOTE-STRING (char-to-string QUOTE-CHAR))

(defun goto-start-of-string ()
  "Assumes that the point is inside a string (not point at a quote).
Move the point to the start of the string, poining at the quote character"
  (while (not (eq (char-under-point) QUOTE-CHAR))
         (backward-char 1)))

(defun goto-end-of-string ()
  "Assumes that the point is inside a string (not point at a quote).
Move the point to the end of the string, poining at the quote character"
  (while (not (eq (char-under-point) QUOTE-CHAR))
         (forward-char 1)))

(defun char-under-point (&optional n)
  "Return the character under point. If n is supplied, it must be a number.
   In that case n is added to point before the character is returned. In that
   way we can access chacters before (-1) or after (1) point too."
  (let* ((n1 (if n n 0))
         (pt1 (+ (point) n1)))
    (string-to-char (buffer-substring pt1 (+ 1 pt1)))))

(defun word-delimitor-scheme ()
  "Return a list of two numbers: the start and the end positions of the Scheme name under point"
 (save-excursion
  (let ((rb (progn (goto-start-of-scheme-word)
                 (point)))
        (re (progn (goto-end-of-scheme-word)
                 (point))))
    (if (and rb re) (list rb re)))))

(defun scheme-name-under-point ()
  (let ((wd (word-delimitor-scheme)))
    (buffer-substring (car wd) (+ 1  (cadr wd)))))


(defun split-string-into-three ()
  "Assume that point is within a string and that point and mark defines a selection. Split the current string into
   three strings: The section before the selection, the selection, and the section after. Leave point at the newly inserted start
   quote of the selected string"
 (let ((qs (char-to-string QUOTE-CHAR)))
  (let ((p1 (min (point) (mark)))
        (p2 (max (point) (mark)))
        (cur-p (point)))
    (goto-char p2) (insert (concat qs " " qs))
    (goto-char p1) (insert (concat qs " " qs))
    (goto-char (+ 2 p1))
)))


; ---------------------------------------------------------------------------------------------------------
; Generally useful LAML commands


(defun nest (function-name)
  "Nest the form designated by the point into a call of function-name. Function-name is a symbol.
When called interactively you are prompted for a symbol in the minibuffer"
  (interactive "SNest into which form: ")
  (insert "(") 
  (insert (symbol-to-string function-name))
  (insert " ")
  (forward-sexp 1)
  (insert ")")
  (backward-sexp 1)
)

(defun unnest ()
  "Unnest the form designated by the point. Point must be at a start or end parenthesis character.
   Leaves point just before the first constituent of the unnested form."
  (interactive)
  (if (= (char-under-point) 41)  ; end parenthesis
      (backward-up-list 1))
  (if (= (char-under-point) 40)  ; start parenthesis
      (let ((p1 (point)))
        (forward-sexp 1)
        (delete-backward-char 1)  ; delete end parenthesis
        (goto-char p1)
        (delete-char 1)  ; delete start parenthesis
        (kill-sexp 1)    ; delete function position
        
      )
  )
)


(defvar concatenate-form nil "The concatenate form to use for LAML")
(setq concatenate-form 'con)

; currently an adhoc collection:
(defvar html-laml-embed-tag-list nil "A list of html and laml tag names (symbols) in which embedding a text string makes sense")
(setq html-laml-embed-tag-list '(em a-tag a-tag-target h kbd img strong pre center cite blockquote i b u p frame box color-frame frame-width))

(defvar laml-single-string-approach t 
  "A boolean variable which defines whether we work with LAML forms which expect a single string as argument.
The alternative is functions which accept an arbitrary list of strings as arguments.")

(defvar smart-word-determination t 
  "Determines whether we automatically select the word in case the point is at the first character in a word")


(defun ensure-concatenation-context ()
  "Point is assumed to be within a string. Make sure that the string is within a direct concatenation context.
If necessary, insert a con form. Con is an alias of string-append in LAML."
  (save-excursion
    (let ((p-cur (point))
          (p-stringstart nil))
      (goto-start-of-string)
      (setq p-stringstart (point))
      (backward-up-list 1) (forward-char 1) 
      (let ((form-name (string-to-symbol (scheme-name-under-point))))  ; only part before dash!
        (if (not (concatenate-formp form-name))
            (progn
               (goto-char p-stringstart)
               (nest concatenate-form)
               )))))
)


(defun concatenate-formp (x)
  (memq x (list concatenate-form)))



(defun word-interpretation ()
  "Return non-nil, if the context signals that the word under point is to be the selection.
Returns true if points happens to be located at the first character in a word"
 (save-excursion
  (backward-char 1) (or (looking-at CR) (looking-at " "))))

(defvar previously-typed-tag "" "The previously typed tag - for reusability.")

(defun embed (tag)
  "Embed the currenly selected string/word into tag (a function call). Ensures automatically that
the function call occurs within a con concatenation context (if laml-single-string-approach is true).
If smart-word-determination is true, a word under point is selected in case point is located at the word's first letter.
The parameter tag must be a symbol"
  (interactive 
    (list (string-to-symbol (completing-read "Embed in which Scheme function: " 
                               (trivial-alist (mapcar (function symbol-to-string) html-laml-embed-tag-list))
                               nil nil previously-typed-tag))))
  (if (and smart-word-determination (word-interpretation)) (mark-word 1))
  (if (looking-at "\"") (exchange-point-and-mark)) ; a trick to avoid problems with a embedding of the rear end of a string.
  (if laml-single-string-approach (ensure-concatenation-context))
  (split-string-into-three)
  (nest tag)
  (remove-empty-siblings)
  (if (mark-marker) (move-marker (mark-marker) (point)))
  (setq previously-typed-tag (symbol-to-string tag))
)

(defvar string-quote-char 34)

; Is there a string to the left of point
(defun string-to-the-left-p ()
  (save-excursion
    (while (blank-char-p (char-under-point))
      (forward-char -1))
    (equal 34 (char-under-point))  ; string quote
  ))

(defun string-to-the-right-p ()
  (save-excursion
    (while (blank-char-p (char-under-point))
      (forward-char +1))
    (equal string-quote-char (char-under-point))  ; string quote
  ))

; assume we are in between to verbatim strings or at the leftmost string quote. Join them.
(defun join-neighbor-strings ()

  ; first join to the left:
  (if (not (equal (char-under-point) string-quote-char))
    (while (not (equal string-quote-char (char-under-point -1)))
       (backward-delete-char-untabify 1)))
  (backward-delete-char-untabify 1) ; delete left string quote

  ; next join to the right:
  (while (not (equal string-quote-char (char-under-point 1)))
     (delete-char 1))
   (delete-char 2) ; delete right quote string and char in front of it.
)

(defun single-con-form-p (x)
  (and (listp x)
       (= (length x) 2)
       (eq 'con (car x))
       (stringp (cadr x))))

; Return the lisp form under and before point. If not possible, return nil.
(defun form-under-point ()
 (save-excursion
   (let ((p (point)))
     (forward-sexp 1)
     (let ((q (point)))
       (if (and p q)
           (let ((form  (car (read-from-string (buffer-substring p q)))))
             form)
           nil)))))

(defun unembed ()
  "Unnest the currently selected form and join neighboring verbatim strings together.
   You are supposed to select a character of the form prefix you want to get rid of, or the start parenthesis."
  (interactive)

  (save-excursion  
    (if (not (looking-at "(")) (backward-up-list 1))
    (if (looking-at "(")
        (unnest))
    ; now in between possible verbatim strings

    ; try joint verbatim strings to the left
    (if (and (string-to-the-left-p) (string-to-the-right-p))
        (join-neighbor-strings))
  
    ; go to end of possibly joined strings
    (while (not (equal string-quote-char (char-under-point)))
       (forward-char 1))
    (forward-char 1)
  
    ; try joint verbatim strings to the right
    (if (and (string-to-the-left-p) (string-to-the-right-p))
        (join-neighbor-strings)
        (forward-char -2))

    ; get rid of redundant concatenate (con ...) form

    ; go to beginning of possibly joined strings
    (while (not (equal string-quote-char (char-under-point)))
       (forward-char -1))
    (forward-char -1)
    (backward-up-list 1)

    ; unnest if we see a con with a single string argument
    (if (single-con-form-p (form-under-point))
        (unnest))
  )
)

; Given some position in a list, return the sibling constituents which are literal empty lists.
; Do not change the position of point.
(defun remove-empty-siblings ()
  (interactive)
  (let ((empty-string-reg-exp "\"\""))
   (save-excursion
    (backward-up-list 1) (forward-char 1)
    (while (not (looking-at ")"))
      (progn
        (if (looking-at empty-string-reg-exp)
            (kill-sexp 1)
            (forward-sexp 1))
        (forward-through-white-space))))))

(defun forward-through-white-space ()
  (skip-through (list 32 10 13)))  ; also include tab


; -----------------------------------------------------------------------------

; Manual templates:

(defun insert-manual-page (form-name)
  (interactive "sName of manual entry: ")
  (insert-lisp-form-0 (make-manual-page form-name) (determine-indentation))

  ; replace the QBXYZ hack: 
  (while (search-forward "(QBXYZ"  nil t)
    (replace-match  "'(" nil t))
  (beginning-of-defun))

(defun make-manual-page (form-name)
  (concat "(manual-page " CR "'" form-name 
     "(title " (string-it form-name) ")"
     "(form "  "(QBXYZ" form-name "))"  ; QBXYZ is a hack. It will be replaced by a quote and a start parenthesis
     "(description " empty-string ")"
     "(parameters"
         "(parameter " empty-string " " empty-string ")"
         "(parameter " empty-string " " empty-string ")" ")"
     "(pre-condition " empty-string ")"
     "(cross-references"
         "(reference " empty-string " " empty-string " " empty-string ")" ")"
     "(misc " empty-string ")" ")"
  ))

(defun insert-manual-section ()
  (interactive)
  (insert-lisp-form-0 (make-manual-section) (determine-indentation)))

(defun make-manual-section ()
  (concat "(manual-section" CR
     "(section-title " empty-string ")"
     "(section-body " empty-string "))"))


(defun insert-manual-internal-tag (kind)
  (interactive  (list (string-to-symbol (completing-read "Insert which kind of internal tag: " 
                      (trivial-alist "title" "form" "pre-condition" "parameter" "example" 
                                      "reference" "internal-references" "misc" "comment" "returns") nil nil)))
    kind))



; ---------------------------------------------------------------------------------------------------
; Elucidator tempaltes
; Currently just simple insertions from the template directory.
; We should consider to use the LAML template stuff.


(defun insert-lispstyle-documentation-entry ()
  "Insert an Eludcidator documentatin entry"
  (interactive)
  (laml-insert-template "documentation-lispstyle-entry"))

(defun insert-lispstyle-documentation-section ()
  "Insert an Eludcidator documentation section"
  (interactive)
  (laml-insert-template "documentation-lispstyle-section"))

(defun insert-documentation-entry ()
  "Insert an Eludcidator documentation entry"
  (interactive)
  (laml-insert-template "documentation-entry"))

(defun insert-documentation-intro ()
  "Insert an Eludcidator documentation entry"
  (interactive)
  (laml-insert-template "documentation-intro"))

(defun insert-documentation-section ()
  "Insert an Eludcidator documentation section"
  (interactive)
  (laml-insert-template "documentation-section"))

; --------------------------------------------------------------

;; Replace <, > and & with characer entities that present these characters properly in HTML.
;; An interactive emacs lisp counterpart to the Scheme function html-protect.
(defun html-protect-buffer ()
  (interactive)
  (goto-char (point-min))
  (replace-string "&"  "&amp;")

  (goto-char (point-min))
  (replace-string "<" "&lt;")

  (goto-char (point-min))
  (replace-string ">" "&gt;")

)